""" Support for mounting a filestorage. 
    Code derived from the original OFS/MountedItem.py which has now disappeared
    from the CVS.

    Use at your own risk. There's probably gods only know special badness 
    awaiting users of this. 
"""

import string, time
import AccessControl.Owned, AccessControl.SecurityManagement, AccessControl.User
import ZODB, ZODB.ZApplication
import Globals, OFS.Application
from App.ApplicationManager import DatabaseManager

from MountedItem import MountedItem

class MountedStorageMixin:
    def _createStorage(self): pass

    def _createDB(self):
	""" Gets the database object, usually by creating a Storage object
        and returning ZODB.DB(storage).
        """
        params = eval(self._params)
        path = params['path']
        create = params['create']
        DB = ZODB.DB(self._createStorage())
        #
        # Excerpt from lib/python/Zope/__init__.py
        #
        # "Log on" as system user
        AccessControl.SecurityManagement.newSecurityManager(
            None, AccessControl.User.system)
        
        # Set up the "application" object that automagically opens
        # connections
        app=ZODB.ZApplication.ZApplicationWrapper(
            DB, 'Application', OFS.Application.Application, (),
            Globals.VersionNameName)
        
        # Initialize products:
        c=app()
        if create:
            obj=c
            for p in string.split(path[1:], '/'):
                if obj._getOb(p,None) is None:
                    from OFS.Folder import Folder
                    folder = Folder()
                    folder.id = p
                    obj._setObject(p, folder)
                    get_transaction().note('Added Folder with id %s' % p)
                    get_transaction().commit()
                obj = obj._getOb(p)

        c._p_jar.close()
        del c

        # "Log off" as system user
        AccessControl.SecurityManagement.noSecurityManager()

        return DB

class MountedStorage(MountedStorageMixin, MountedItem):
    '''A MountPoint with a basic interface for displaying the
    reason the database did not connect.
    '''

    meta_type = 'MountedStorage'
    
    icon = 'p_/broken'
    manage_options = ({'label':'Traceback', 
                       'action':'manage_traceback'},)
    manage_traceback = Globals.HTMLFile('dtml/mountfail', globals())

    def __init__(self, id, title='', path=None, create=None, params={}):
        self.title = title
        # check path
        if path is None or path == '' or path == '/':
            path = '/'
        if path[0] != '/':
            path = '/' + path
        if len(path) > 1 and path[len(path) - 1] != '/':
            path = path + '/'
        # add id
        path = path + id
        # update params
        params.update({'path':path,'create':create})
        # MountedItem
        MountedItem.__init__(self,path,params)

class MountManager(MountedStorageMixin, DatabaseManager):
    '''A Manager for a Mounted Storage
    '''

    meta_type = 'MountedStorageManager'

    manage_options=(
        (
        {'label':'Database', 'action':'manage_main',
         'help':('OFSP','Database-Management_Database.stx')},
        #{'label':'Cache Parameters', 'action':'manage_cacheParameters',
        # 'help':('OFSP','Database-Management_Cache-Parameters.stx')},
        #{'label':'Flush Cache', 'action':'manage_cacheGC',
        # 'help':('OFSP','Database-Management_Flush-Cache.stx')},
        )
        )

    def __init__(self, id, title='', params={}):
        self.id = id
        self.title = title
        self._params = repr(params)

    def _getDB(self):
        return ZODB.DB(self._createStorage())

    def db_name(self):
        return self._getDB().getName()

    def db_size(self):
        s=self._getDB().getSize()
        if type(s) is type(''): return s

        if s >= 1048576.0: return '%.1fM' % (s/1048576.0)
        return '%.1fK' % (s/1024.0)
